################################################################################
#       A PROGRAM FOR FITTING A LATENT CLASS TRAJECTORY ANALYSIS MODEL         #
#       Dorte Vistisen, Steno Diabetes Center, February 25th 2015              #
################################################################################


#The hlme() function from the lcmm package is used for gaussian outcomes - alternatively, use lcmm() for non-linear models
#The hlme() function needs complete data and character variables needs to expressed as indicator variables
#The number of classes needs to be specified a priori, so try out different number of classes and select the model with the lowest BIC 





##### DATA  #####
library(foreign)                                                                #package for reading in SAS export file
library(lcmm)                                                                   #package for latent class trajectory analysis: http://cran.r-project.org/web/packages/lcmm/lcmm.pdf

all <- read.xport( "C:/..../all.xpt" )                                          #reading in SAS export file
names(all) <- tolower( names(all) )
attach(all)
names(all)

all$sex <- as.factor(all$sex)                                                   #defining factor variables
  
all <- transform(all,time2=time^2,time3=time^3)                                 #creating quadratic and cubic time variables


##### MODEL #####     
datasub <- all[complete.cases(all[,c("y","time","age","sex")]),]                #hlme() requires complete cases 
  

###3rd degree poly###   
fitpoly <- hlme(y ~ time + time2 + time3                                        #y (continuous) as function of time (3rd degree poly) and adjusting for age and sex
            + age + sex,                                      
            mixture =~ time + time2 + time3,                                    #terms that can vary between latent classes
            random =~ time,                                                     #random effect
            subject = 'id', ng = 3, data = datasub)                             #testing 3 classes

fitpoly                                                                         #mode fit: including Baysian Information Criterion(BIC) - low is good!
postprob(fitpoly)                                                               #average class probabilities by latent classes

x <- seq(floor(min(datasub$time)),0,by=1)                                       #range of time in original data
plotdata <- data.frame(cbind(1,x,x^2,x^3,60+x,1))                               #creating prediction dataset
names(plotdata) <- fitpoly$Xnames    

outpoly <- plot.predict.hlme(fitpoly,plotdata,var.time="time",                  #plot of class-specific curves
  legend.loc="topleft")

datasub <- merge(datasub, fitpoly$pprob[,1:2], by ="id")                        #merging class to data


  



###or alternatively splines###
MM <- with(datasub, bs(time,df=5,Boundary.knots=c(-10,1)) )                     
DD <- detrend( MM, datasub$time )
colnames(DD) <- paste("x",colnames(DD),sep="")
datasub <- cbind( datasubf, DD )
names(datasub)
DD[1:10,]

fitspl <- hlme(y ~ x1 + x2 + x3 + x4 + time 
            + age + sex,                                     
            mixture =~ x1 + x2 + x3 + x4 + time,
            random =~ time,
            subject = 'id', ng = 3, data = datasub)

fitspl
postprob(fitspl)

x <- seq(floor(min(datasub$time)),0,by=1)                                       #range of time
MMx <- bs(x,df=5,Boundary.knots=c(-10,1))                                       #spline of x
DDx <- detrend( MMx, x )
plotdata <- data.frame(cbind(1,DDx,x,60+x,1))                           
names(plotdata) <- fitspl$Xnames

outspl <- plot.predict.hlme(fitspl,plotdata,var.time="time",
  legend.loc="topleft")

datasub <- merge(datasub, fitspl$pprob[,1:2], by ="id")                         #merging class to data












