### R code from vignette source 'read.rnw'
### Encoding: UTF-8

###################################################
### code chunk number 1: read.rnw:2-7
###################################################
options( width=90,
#        prompt=" ", continue=" ", # Makes it easier for students to
                                   # copy from the final pdf document
         SweaveHooks=list( fig=function()
         par(mar=c(3,3,1,1),mgp=c(3,1,0)/1.6,las=1,bty="n" ) ) )


###################################################
### code chunk number 2: read.rnw:22-28
###################################################
grnam <- function( pat, dfr, verbose=TRUE )
{ 
wh <- grep( pat, names(dfr) ) 
if( verbose ) print( names(dfr)[wh] )
return(wh)
}


###################################################
### code chunk number 3: read.rnw:33-39
###################################################
maxlab <- function( x )
    {
    tt <- table(x)
    tt <- tt[names(tt)!=""]
    names( tt )[tt==max(tt)]
    }


###################################################
### code chunk number 4: read.rnw:46-62
###################################################
library( readstata13 )
library( Epi )
nef <- read.dta13( "./data/nephrohkworkdata.dta", nonint.factors=TRUE )
wh <- grnam( "enhed", nef )
for( i in wh ) {
   tt <- table( nef[,i], exclude=NULL )
   if( length(tt)>3 ){ 
     cat( "\n",names(nef)[i],":" )
     print( tt ) }
               }
units <- sapply( nef[wh], maxlab )
names( units ) <- gsub("_enhed","",names(units) )
cbind( units )
nef <- nef[,-wh]
# check that we have units of actual variables in nef
match( names(units), names(nef) )


###################################################
### code chunk number 5: read.rnw:74-76
###################################################
wh <- grnam( "^d", nef )
wh <- wh[c(2:6,8:10)]


###################################################
### code chunk number 6: read.rnw:81-91
###################################################
old <- c("dob","debut_diabetes","d_stenostart","date",
         "d_renaldisease","d_esrd","d_stenoslut","d_dth",
         "renaldisease")
new <- c("dob","doDM","doin","dolab",
         "dockd","doesrd","dox","dodth",
         "ckd")
wh <- match( old, names(nef) )
cbind( names( nef )[wh],   new )
       names( nef )[wh] <- new
names( nef )


###################################################
### code chunk number 7: cal.yr
###################################################
nef <- cal.yr( nef )
nef$dodm <- nef$doDM
class( nef$dodm ) <- class( nef$doDM ) <- class( nef$dob )


###################################################
### code chunk number 8: read.rnw:107-109
###################################################
wh <- grnam( "^do", nef )
summary( nef[,wh] )


###################################################
### code chunk number 9: read.rnw:112-114
###################################################
subset( nef, dolab < 1930 )
nef <- subset( nef, dolab > 1930)


###################################################
### code chunk number 10: read.rnw:118-122
###################################################
tt <- with( nef, table(newid,is.na(doDM)) )
dim(tt)
range( apply( tt>0, 1, sum ) )
apply( tt>0, 2, sum )


###################################################
### code chunk number 11: read.rnw:135-147
###################################################
na.chk <- function( var )
    {
    tt <- table( nef$newid, is.na(nef[,var] ) )
    print( sum( apply( tt>0, 1, sum ) > 1 ) )
    invisible( tt )
    }
t.ren <- na.chk("dockd")
t.esr <- na.chk("doesrd")
t.dth <- na.chk("dodth")
t.dm <- na.chk("doDM")
t.in <- na.chk("doin")
t.ex <- na.chk("dox")


###################################################
### code chunk number 12: read.rnw:152-155
###################################################
summary( with( nef, tapply( doDM, newid, var, na.rm=TRUE ) ) )
summary( with( nef, tapply( doin, newid, var, na.rm=TRUE ) ) )
summary( with( nef, tapply( dox , newid, var, na.rm=TRUE ) ) )


###################################################
### code chunk number 13: read.rnw:160-163
###################################################
wh <- sort( c(1,8,10,12,wh) )
head( nef[nef$newid==44,wh] )
tail( nef[nef$newid==44,wh] )


###################################################
### code chunk number 14: miNA
###################################################
miNA <- function(x) if( all(is.na(x)) ) NA else min( x, na.rm=TRUE )
for( vv in c("doesrd","dockd","dodth") )
   nef[,vv] <- ave( nef[,vv], nef$newid, FUN = miNA )
head( nef[nef$newid==44,wh] )
tail( nef[nef$newid==44,wh] )


###################################################
### code chunk number 15: read.rnw:201-204
###################################################
nef$dodm <- ifelse( is.na(nef$doDM), 
                    ave( nef$dolab, nef$newid, FUN=min ) - 1/4,
                    nef$doDM )


###################################################
### code chunk number 16: read.rnw:211-214
###################################################
nrow( nef )
nef <- subset( nef, dolab>2001 )
nrow( nef )


###################################################
### code chunk number 17: read.rnw:238-242
###################################################
with( nef, cbind(
           table( dolab==dockd , ckd , exclude=NULL ),
           table( dolab==doesrd, esrd, exclude=NULL ),
           table( dolab==dodth , dth , exclude=NULL ) ) )


###################################################
### code chunk number 18: read.rnw:244-249
###################################################
( zz <- subset( nef, (dolab==dockd  &  ckd==0) |  
                     (dolab==doesrd & esrd==0) |  
                     (dolab==dodth  &  dth==0) )[,wh] )
fishy <- subset( nef, ( dolab==doesrd | dolab==dodth ) & newid %in% zz$newid )
for( ii in zz$newid ) print( subset(fishy,newid==ii) )


###################################################
### code chunk number 19: read.rnw:254-257
###################################################
table( duplicated(nef[,c("newid","dolab")]), ckd  =nef$ckd  )
table( duplicated(nef[,c("newid","dolab")]), esrd =nef$esrd )
table( duplicated(nef[,c("newid","dolab")]), death=nef$dth  )


###################################################
### code chunk number 20: aggr
###################################################
# where is the key (names in kn) (numeric)
ix <- match( kn <- c("newid","dolab"), names(nef) )
# names of numeric variables, except key
wh <- names( which( sapply( nef[,-ix], is.numeric ) ) )
# names of non-numeric variables, except key
hw <- setdiff( names(nef), c(wh,kn) )
# average over non-missing within key
system.time( an <- aggregate( nef[,wh], nef[,kn],
                              FUN = mean, na.rm=TRUE ) )
# reset the integers for ckd, esrd and death:
an[,c("ckd","esrd","dth")] <- ( an[,c("ckd","esrd","dth")] > 0 )*1
# the first value if the non-numerical
af <- nef[!duplicated(nef[,kn]),c(kn,hw)]
nrow( nef )
nrow( an )
nrow( af )
intersect( names(af), names(an) )
nef <- merge( af, an )
nrow( nef )


###################################################
### code chunk number 21: read.rnw:286-300
###################################################
# only one record per person
wh <- grnam( "^do", nef )
np <- nef[!duplicated(nef$newid),wh]
# diabetes before birth?
with( np, table( doDM >= dob, exclude=NULL ) ) 
subset( np, doDM < dob )
# renal disease before DM?
with( np, table( dockd >= doDM, exclude=NULL ) ) 
# ESRD before DM?
with( np, table( doesrd >= doDM, exclude=NULL ) ) 
# ESRD before renal disease =
with( np, table( doesrd >= dockd, exclude=NULL ) ) 
# Death after any type of event ?
with( np, table( dodth >= pmax(doDM,dockd,doesrd,na.rm=TRUE) ) )


###################################################
### code chunk number 22: read.rnw:304-305
###################################################
nef <- transform( nef, doDM = ifelse( doDM<dob, dob+1/4, doDM) )


###################################################
### code chunk number 23: read.rnw:309-315
###################################################
tt <- table( np$doin )
tt[tt==max(tt)]
as.Date.cal.yr( mostin <- as.numeric(names(tt[tt==max(tt)])) )
sort( np$doin )[1:10]
nef$doin <- pmax( nef$doin, mostin )
summary( nef$doin )


###################################################
### code chunk number 24: read.rnw:318-319 (eval = FALSE)
###################################################
## save( nef, file="./data/nef.Rda" )


###################################################
### code chunk number 25: dates
###################################################
clear()
load( file="./data/nef.Rda" )
nuf <- nef[,c("dob",
              "doDM", 
              "dodm", 
              "doin", 
              "dockd",
              "doesrd",
              "dodth",
              "dox")]
dim( nef )
dim( nuf )
nuf <- nuf[!duplicated(nuf),]
dim( nuf )
hh <- 
function( x, lab, ... ) hist(x, col="black", main="", xlab=lab, ylab="", ... )
par( mfrow=c(3,3), mar=c(3,3,1,1), mgp=c(3,1,0)/1.6, las=1 )
hh( nuf$dob   , "Date of birth"         , breaks=seq(1900,2016,1   ) )
hh( nuf$doDM  , "Date of diabetes debut", breaks=seq(1930,2016,1   ), ylim=c(0,600) )
hh( nuf$dodm  , "Amended diabetes debut", breaks=seq(1930,2016,1   ), ylim=c(0,600) )
hh( nef$dolab , "Date of visit to SDC"  , breaks=seq(2000,2016,1/12) ) ; abline(v=2000:2015,col="red")
hh( nuf$dockd , "Date of CKD"           , breaks=seq(1979,2016,1/ 2) ) ; axis(side=1,at=1980:2016,labels=NA,tck=-0.01)
hh( nuf$doesrd, "Date of ESRD"          , breaks=seq(1979,2016,1/ 2) ) ; axis(side=1,at=1980:2016,labels=NA,tck=-0.01)
hh( nuf$dodth , "Date of death"         , breaks=seq(2000,2016,1/12) ) ; abline(v=2000:2015,col="red")
hh( nuf$doin  , "Date of entry at SDC"  , breaks=seq(1993,2016,1/12), ylim=c(0,200) ) ; abline(v=1993:2015,col="red")
hh( nuf$dox  , "Date of exit from SDC"  , breaks=seq(1993,2016,1/12), ylim=c(0,200) ) ; abline(v=1993:2015,col="red")


###################################################
### code chunk number 26: read.rnw:365-368
###################################################
with( nuf, table( has.in = !is.na(doin),
                  has.ex = !is.na(dox), exclude=NULL ) )
range( nuf$dox, na.rm=TRUE )


###################################################
### code chunk number 27: inout
###################################################
with( nuf, plot( ifelse( doin<1993.754, 1993.5-runif(nrow(nuf)), doin ),
                 pmin( dox, 2015.3+runif(nrow(nuf)), na.rm=TRUE ),
                 xlab="Date of entry to SDC",
                 ylab="Date of exit from SDC",
                 pch=16, cex=0.3 ) )
for( i in 0:2 ) abline( i, 1, col="red" )
rug( 2013+0:2/2, side=2 )


###################################################
### code chunk number 28: recs
###################################################
addmargins( with( nef, table( gfr=!is.na(gfr), egfr=!is.na(egfr) ) ) )

par( mfrow=c(1,2), mar=c(3,3,1,1), mgp=c(3,1,0)/1.6, yaxs="i", las=1, bty="n" )

nt <- with( nef, table(table(newid)) )
plot( with( nef, nt ), type="h", lwd=5, xaxt="n", ylim=c(0,500), xlim=c(0,150),
      ylab="No. persons", xlab="No. records per person" )
axis(side=1)
axis(side=1,at=1:25*10,labels=NA)

nt <- with( subset( nef, !is.na(egfr) | !is.na(gfr) ), table(table(newid)) )
plot( with( nef, nt ), type="h", lwd=5, xaxt="n", ylim=c(0,500), xlim=c(0,150),
      ylab="No. persons", xlab="No. records with (e)GFR per person" )
axis(side=1)
axis(side=1,at=1:25*10,labels=NA)
many <- nt[nt>500]
names( many )
for(i in 1:length(many)) text( 10+20*i, 490,
                               paste(names(many)[i],"\n",many[i]), adj=1 )


###################################################
### code chunk number 29: read.rnw:419-419
###################################################



###################################################
### code chunk number 30: read.rnw:426-428
###################################################
with( nef, table(eGFR=!is.na(egfr),GFR=!is.na(gfr)) )
with( nef, table(ucr=!is.na(ualbcrea),dualb=!is.na(dualb)) )


###################################################
### code chunk number 31: read.rnw:433-444
###################################################
nef <- transform( nef, GFR =      pmin( egfr, gfr, na.rm=TRUE ),
                    ren.st = cut( pmin( egfr, gfr, na.rm=TRUE ),
                                  breaks=c(0,15,30,45,60,90,Inf),
                                  include.lowest=TRUE ),
                    alb.st = cut( pmax(dualb,ualbcrea,na.rm=TRUE),
                                  breaks=c(0,30,300,Inf),
                                  right=FALSE ) )
non.miss <- function(x) sum(x[-length(x)])
with( nef, addmargins( table( ren.st, alb.st, useNA="ifany" ),
                       FUN=list(list(sum,non.miss),list(sum,non.miss)), 
                     quiet=TRUE ) )[c(1:6,9,7,8),c(1:3,6,4,5)]


###################################################
### code chunk number 32: read.rnw:447-448
###################################################
save( nef, file="./data/nef.Rda" )


